/*
 *  GnomeZip
 *  filesel.c: GnomeZip's file selector
 *
 *  Copyright (C) 2000 Hongli Lai
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _FILESEL_C_
#define _FILESEL_C_

#include <filesel.h>
#include <gnome.h>
#include <misc.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*********  Signals  *********/

enum
{
    SELECT,
    CANCEL,
    LAST_SIGNAL
};

static gint filesel_signals[LAST_SIGNAL] = {0, 0};

typedef void (*GzFileSelSelectSignal)
             (GzFileSel *filesel,
              gchar *selected_file,
              GList *selected_files,
              gpointer user_data);

static GtkFileSelectionClass *parent_class;
static void gz_file_sel_destroy (GtkObject *obj);

/*********  Functions ********/


static void
gz_file_sel_select_signal (GtkObject *object, GtkSignalFunc func,
  gpointer func_data, GtkArg *args)
{
    GzFileSelSelectSignal rfunc;

    rfunc = (GzFileSelSelectSignal) func;

    (*rfunc) ((GzFileSel *) object, GTK_VALUE_STRING (args[0]),
      GTK_VALUE_POINTER (args[1]), func_data);
}


/*********  Functions  *********/


static gboolean
gz_file_sel_on_delete_event (GtkWidget *filesel, gpointer user_data)
{
    gtk_widget_hide (filesel);
    return TRUE;
}


static GtkWidget *
gz_file_sel_create_pixmap_button (GzFileSel *filesel, GtkToolbar *toolbar,
  const gchar *label_text, const gchar *tooltip, char **xpm_data,
  GtkSignalFunc func)
{
    GtkWidget *button;
    GtkWidget *pixmap_widget = NULL;
    GdkPixmap *pixmap;
    GdkBitmap *mask = NULL;

    if (xpm_data != NULL)
    {
        pixmap = gdk_pixmap_create_from_xpm_d (GTK_WIDGET (filesel)->window,
          &mask, NULL, xpm_data);
        pixmap_widget = gtk_pixmap_new (pixmap, mask);
    }

    button = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_BUTTON,
      NULL, label_text, tooltip, NULL, pixmap_widget, NULL, NULL);
    if (func != NULL)
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
          func, GTK_OBJECT (filesel));

    return button;
}

static GtkWidget *
gz_file_sel_create_pixmap_button_stock (GzFileSel *filesel,
  GtkToolbar *toolbar, const gchar *label_text, const gchar *tooltip,
  const gchar *stock, GtkSignalFunc func)
{
    GtkWidget *button;
    GtkWidget *pixmap_widget = NULL;

    pixmap_widget = gnome_stock_pixmap_widget (GTK_WIDGET (filesel), stock);
    button = gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_BUTTON,
      NULL, label_text, tooltip, NULL, pixmap_widget, NULL, NULL);
    if (func != NULL)
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
          func, GTK_OBJECT (filesel));

    return button;
}


static void gz_file_sel_on_create_dir_click (GzFileSel *filesel);
static void gz_file_sel_on_refresh_click (GzFileSel *filesel);

static void
gz_file_sel_on_create_dir (gchar *string, gpointer data)
{
    gchar *path;

    if (string == NULL) return;

    path = g_strdup_printf ("%s%s",
      gtk_file_selection_get_filename (GTK_FILE_SELECTION (data)),
      string);

    if (mkdir (path, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) == 0)
        gz_file_sel_on_refresh_click (GZ_FILE_SEL (data));
    else {
        gchar *msg = "";
        gchar *s;

        switch (errno)
        {
        case EEXIST:
            msg = _("The directory already exists.");
            break;
        case EFAULT:
            msg = _("The pathname points outside your accessible address"
                    " space.");
            break;
        case EACCES:
            msg = _("You don't have the permission to create it.");
            break;
        case ENAMETOOLONG:
            msg = _("The pathname is too long.");
            break;
        case EROFS:
            msg = _("The filesystem is read-only.");
            break;
        case ENOSPC:
            msg = _("There's no space left on this device.");
            break;
        }

        s = g_strdup_printf (_("Unable to create directory.\n%s"), msg);
        gnome_error_dialog (s);
        g_free (s);
    }
    g_free (path);
    
}


static void
gz_file_sel_on_create_dir_click (GzFileSel *filesel)
{
    gnome_request_dialog (FALSE, _("Enter a name for this directory:"),
      _("New directory"), PATH_MAX, gz_file_sel_on_create_dir, filesel,
      GTK_WINDOW (filesel));
}


static void
gz_file_sel_on_delete (gint reply, gpointer data)
{
    gint i;
    GtkCList *clist = GTK_CLIST (GTK_FILE_SELECTION (data)->file_list);

    if (reply != 0) return;

    for (i = 0; i < g_list_length (clist->row_list); i++)
    {
        GtkCListRow *row = g_list_nth_data (clist->row_list, i);
        if (row->state == GTK_STATE_SELECTED &&
            remove (row->cell->u.text) == -1)
        {
            gchar *msg = "";
            gchar *s;

            switch (errno)
            {
            case EFAULT:
                msg = _("The pathname points outside your accessible address"
                        " space.");
                break;
            case EACCES:
                msg = _("You don't have the permission to delete this file.");
                break;
            case EROFS:
                msg = _("The filesystem is read-only.");
                break;
            }

            s = g_strdup_printf (_("Unable to delete file\n%s\n%s"), msg,
              row->cell->u.text);
            gnome_error_dialog (s);
            g_free (s);
        }
    }
    gz_file_sel_on_refresh_click (GZ_FILE_SEL (data));
}


static void
gz_file_sel_on_delete_click (GzFileSel *filesel)
{
    GtkCList *clist = GTK_CLIST (GTK_FILE_SELECTION (filesel)->file_list);
    gint i;
    gboolean sel = FALSE;

    for (i = 0; i < g_list_length (clist->row_list); i++)
    {
        GtkCListRow *row = g_list_nth_data (clist->row_list, i);
        if (row->state == GTK_STATE_SELECTED)
        {
            sel = TRUE;
            break;
        }
    }

    if (sel == TRUE)
        gnome_question_dialog_parented (_("Are you sure you want to delete "
          "the selected file(s)?"), gz_file_sel_on_delete, filesel,
          GTK_WINDOW (filesel));
    else
        gnome_error_dialog (_("No files are selected."));
}


static void
gz_file_sel_on_home_click (GzFileSel *filesel)
{
    gtk_file_selection_complete (GTK_FILE_SELECTION (filesel), "~/");
}


static void
gz_file_sel_on_refresh_click (GzFileSel *filesel)
{
    gtk_file_selection_complete (GTK_FILE_SELECTION (filesel), "./");
}


static void
gz_file_sel_on_rename (gchar *string, GList *list)
{
    gchar *fn, *dir, *oldfn, *newfn;
    GzFileSel *filesel;

    fn = (gchar *) g_list_nth_data (list, 0);
    if (string == NULL || g_strcasecmp (string, fn) == 0) return;

    dir = (gchar *) g_list_nth_data (list, 1);
    filesel = (GzFileSel *) g_list_nth_data (list, 2);
    oldfn = g_strdup_printf ("%s/%s", dir, fn);
    newfn = g_strdup_printf ("%s/%s", dir, string);

    if (rename (oldfn, newfn) == -1)
    {
        gchar *errmsg, *msg;

        switch (errno)
        {
        case EACCES: errmsg = "permission denied.";
        case ENAMETOOLONG: errmsg = "filename too long.";
        case EROFS: errmsg = "filesystem is read-only.";
        case ENOSPC: errmsg = "there's not enough free space on this device.";
        default: errmsg = "Unknown error.";
        }

        msg = g_strdup_printf ("An error occured while trying to rename the file:"
          "\n%s", errmsg);
        gnome_error_dialog (msg);
        g_free (msg);
    } else
        gz_file_sel_on_refresh_click (filesel);
    g_free (oldfn);
    g_free (newfn);
}


static void
gz_file_sel_on_rename_click (GzFileSel *filesel)
{
    GtkCList *fl;
    gchar *fn = NULL;
    gint i;

    fl = (GtkCList *) GTK_FILE_SELECTION (filesel)->file_list;
    for (i = 0; i < g_list_length (fl->row_list); i++)
    {
        GtkCListRow *row = g_list_nth_data (fl->row_list, i);
        if (row->state == GTK_STATE_SELECTED)
        {
            fn = row->cell->u.text;
            break;
        }
    }

    if (fn)
    {
        GList *list = NULL;

        list = g_list_append (list, fn);
        list = g_list_append (list, GTK_LABEL (GTK_BIN (GTK_FILE_SELECTION (
          filesel)->history_pulldown)->child)->label);
        list = g_list_append (list, filesel);
        gnome_request_dialog (FALSE, _("Rename this file to:"),
          fn, PATH_MAX, (GnomeStringCallback) gz_file_sel_on_rename,
          (gpointer) list, GTK_WINDOW (filesel));
    } else
        gnome_error_dialog (_("No files are selected."));
}


static void
gz_file_sel_on_xterm_click (GzFileSel *filesel)
{
    /* The history pulldown */
    GtkLabel *h = GTK_LABEL (GTK_BIN (GTK_FILE_SELECTION (
      filesel)->history_pulldown)->child);
    /* Get the current directory */
    gchar *dir = g_strdup (h->label);
    /* the current workdirectory */
/*    gchar *wd = g_new0 (gchar, PATH_MAX + 50);

    wd = getcwd (wd, sizeof (wd) - 1);
    chdir (dir);
    system ("xterm &");
    chdir (wd);
    g_free (wd); */
    gnome_execute_shell (dir, "xterm");
    g_free (dir);
}


static gboolean
gz_file_sel_on_ok_click (GzFileSel *filesel)
{
    gchar *dir = g_strdup (GTK_LABEL (GTK_BIN (GTK_FILE_SELECTION (
      filesel)->history_pulldown)->child)->label);
    GtkCList *clist;
    guint i;

    if (filesel->selection_type == GZ_FILE_SEL_TYPE_FILE_ONLY)
    {
        gchar *fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
        if (g_file_test (fn, G_FILE_TEST_ISDIR) == TRUE)
        {
            gnome_error_dialog (_("Error: you cannot select directories.\n"
                                  "Please select a file."));
            return TRUE;
        }
    }

    
    /* Free any previous filenames */
    if (filesel->filename != NULL) g_free (filesel->filename);

    if (g_list_length (filesel->files) > 1)
    {
        for (i = g_list_length (filesel->files) - 1; i >= 0; i--)
        {
            gchar *tmp = (gchar *) g_list_nth_data (filesel->files, i);

            g_list_remove (filesel->files, tmp);
            g_free (tmp);
        }

        g_list_free (filesel->files);
    } else if (g_list_length (filesel->files) > 0)
    {
        g_free (g_list_nth_data (filesel->files, 0));
        g_list_free (filesel->files);
    }
    filesel->files = NULL;


    if (filesel->selection_type == GZ_FILE_SEL_TYPE_DIR_ONLY)
        clist = GTK_CLIST (GTK_FILE_SELECTION (filesel)->file_list);
    else
        clist = GTK_CLIST (GTK_FILE_SELECTION (filesel)->dir_list);
    {
        GList *filelist = NULL;
        guint selcount = 0;

        for (i = 0; i < g_list_length (clist->row_list); i++)
        {
            GtkCListRow *row = (GtkCListRow *) g_list_nth_data (clist->row_list, i);
            if (row->state == GTK_STATE_SELECTED)
            {
                filelist = g_list_append (filelist, row);
                selcount++;
            }
        }

        if (selcount > 1)
        {
            GtkCListRow *row;

            for (i = 0; i < g_list_length (filelist); i++)
            {
                gchar *tmp;

                row = (GtkCListRow *) g_list_nth_data (filelist, i);
                tmp = g_strdup_printf ("%s/%s", dir, row->cell->u.text);
                filesel->files = g_list_append (filesel->files, (gpointer) tmp);
            }

            row = (GtkCListRow *) g_list_nth_data (filelist, 0);
            filesel->filename = g_strdup_printf ("%s/%s", dir, row->cell->u.text);
        } else
        {
            filesel->filename = g_strdup (gtk_file_selection_get_filename (
              GTK_FILE_SELECTION (filesel)));
            filesel->files = g_list_append (filesel->files,
              (gpointer) g_strdup (filesel->filename));
        }
    }


    g_free (dir);
    gtk_widget_hide (GTK_WIDGET (filesel));
    gtk_signal_emit (GTK_OBJECT (filesel), filesel_signals[SELECT],
      filesel->filename, filesel->files);
    return FALSE;
}


static void
gz_file_sel_emit_cancel_signal (GzFileSel *filesel, gpointer user_data)
{
    gtk_signal_emit (GTK_OBJECT (filesel), filesel_signals[CANCEL]);
}


static void
gz_file_sel_init (GzFileSel *filesel)
{
    #include <gnome-terminal.xpm>
    GtkWidget *main_vbox = GTK_FILE_SELECTION (filesel)->main_vbox;

    gtk_widget_realize (GTK_WIDGET (filesel));
    gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (filesel));
    gz_file_sel_set_selection_type (filesel, GZ_FILE_SEL_TYPE_FILE_ONLY);
    gtk_container_remove (GTK_CONTAINER (GTK_FILE_SELECTION (filesel)->main_vbox),
      GTK_FILE_SELECTION (filesel)->button_area);


    /* Create the file operation bar */
    {
        GtkWidget *handlebox;
        GtkWidget *toolbar;

        handlebox = gtk_handle_box_new ();
        gtk_container_set_border_width (GTK_CONTAINER (handlebox), 1);

        toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
          GTK_TOOLBAR_ICONS);
        gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar), 16);
        gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
        gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), GTK_RELIEF_NONE);
        gtk_container_set_border_width (GTK_CONTAINER (toolbar), 1);
        gtk_container_add (GTK_CONTAINER (handlebox), toolbar);

        gz_file_sel_create_pixmap_button_stock (filesel, GTK_TOOLBAR (toolbar),
          _("New Dir"), _("Create a new directory in the current directory"),
          GNOME_STOCK_PIXMAP_NEW, gz_file_sel_on_create_dir_click);
        gz_file_sel_create_pixmap_button_stock (filesel, GTK_TOOLBAR (toolbar),
          _("Delete"), _("Delete the selected file(s)"),
          GNOME_STOCK_PIXMAP_TRASH, GTK_SIGNAL_FUNC (gz_file_sel_on_delete_click));
        gz_file_sel_create_pixmap_button_stock (filesel, GTK_TOOLBAR (toolbar),
          _("Rename"), _("Rename the selected file"),
          GNOME_STOCK_PIXMAP_BOOK_OPEN, GTK_SIGNAL_FUNC (gz_file_sel_on_rename_click));
        gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
        gz_file_sel_create_pixmap_button_stock (filesel, GTK_TOOLBAR (toolbar),
          _("Home"), _("Browse to your home directory"),
          GNOME_STOCK_PIXMAP_HOME, GTK_SIGNAL_FUNC (gz_file_sel_on_home_click));
        gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
        gz_file_sel_create_pixmap_button_stock (filesel, GTK_TOOLBAR (toolbar),
          _("Refresh"), _("Refresh the current directory"),
          GNOME_STOCK_PIXMAP_REFRESH, GTK_SIGNAL_FUNC (gz_file_sel_on_refresh_click));
        gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
        gz_file_sel_create_pixmap_button (filesel, GTK_TOOLBAR (toolbar),
          _("Xterm"), _("Lauch a terminal"),
          gnome_terminal_xpm, GTK_SIGNAL_FUNC (gz_file_sel_on_xterm_click));

        gtk_box_pack_start (GTK_BOX (main_vbox), handlebox, FALSE, FALSE, 0);
        gtk_box_reorder_child (GTK_BOX (main_vbox), handlebox, 0);
        gtk_widget_show_all (handlebox);
        gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
    }


    gtk_signal_connect (GTK_OBJECT (filesel), "delete_event",
      GTK_SIGNAL_FUNC (gz_file_sel_on_delete_event), NULL);
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
      "clicked", GTK_SIGNAL_FUNC (gz_file_sel_on_ok_click), GTK_OBJECT (filesel));
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
      "clicked", GTK_SIGNAL_FUNC (gtk_widget_hide), GTK_OBJECT (filesel));

    gtk_signal_connect (GTK_OBJECT (filesel), "destroy",
      GTK_SIGNAL_FUNC (gz_file_sel_emit_cancel_signal), NULL);
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
      "clicked", GTK_SIGNAL_FUNC (gz_file_sel_emit_cancel_signal), GTK_OBJECT (filesel));
}


static void
gz_file_sel_class_init (GzFileSelClass *klass)
{
    GtkObjectClass *object_class = (GtkObjectClass *) klass;

    filesel_signals[SELECT] = gtk_signal_new (
      "select",
      GTK_RUN_FIRST,
      object_class->type,
      GTK_SIGNAL_OFFSET (GzFileSelClass, select),
      gz_file_sel_select_signal,
      GTK_TYPE_NONE,
      2, GTK_TYPE_STRING, GTK_TYPE_POINTER
    );
    filesel_signals[CANCEL] = gtk_signal_new (
      "cancel",
      GTK_RUN_FIRST,
      object_class->type,
      GTK_SIGNAL_OFFSET (GzFileSelClass, cancel),
      gtk_signal_default_marshaller,
      GTK_TYPE_NONE,
      2, GTK_TYPE_POINTER, GTK_TYPE_POINTER
    );

    gtk_object_class_add_signals (object_class, filesel_signals, LAST_SIGNAL);
    klass->select = NULL;
    klass->cancel = NULL;
    object_class->destroy = (void(*)(GtkObject *object)) gz_file_sel_destroy;

    parent_class = gtk_type_class (gtk_file_selection_get_type ());
}


guint
gz_file_sel_get_type (void)
{
    static guint ga_type = 0;

    if (!ga_type) {
        GtkTypeInfo ga_info = {
          "GzFileSel",
          sizeof (GzFileSel),
          sizeof (GzFileSelClass),
          (GtkClassInitFunc) gz_file_sel_class_init,
          (GtkObjectInitFunc) gz_file_sel_init,
          (GtkArgSetFunc) NULL,
          (GtkArgGetFunc) NULL,
          (GtkClassInitFunc) NULL
        };
        ga_type = gtk_type_unique (gtk_file_selection_get_type (), &ga_info);
    }
    return ga_type;
}


GzFileSel *
gz_file_sel_new (const gchar *title)
{
    GzFileSel *gz_fs = GZ_FILE_SEL (gtk_type_new ((GtkType) GZ_TYPE_FILE_SEL));
    gtk_window_set_title (GTK_WINDOW (gz_fs), g_strdup (title));
    return gz_fs;
}

static void
gz_file_sel_destroy (GtkObject *obj)
{
    gint i;
    GzFileSel *fs;

    g_return_if_fail (obj != NULL);
    g_return_if_fail (GZ_IS_FILE_SEL (obj));

    fs = (GzFileSel *) obj;
    g_free (fs->filename);
    fs->filename = NULL;

    if (g_list_length (fs->files) > 0)
    for (i = g_list_length (fs->files) - 1; i >= 0; i--)
    {
        gchar *tmp = (gchar *) g_list_nth_data (fs->files, i);
        g_free (tmp);
    }
    g_list_free (fs->files);
    fs->files = NULL;

    if (GTK_OBJECT_CLASS (parent_class)->destroy)
        (* GTK_OBJECT_CLASS (parent_class)->destroy) (obj);
}


gchar *
gz_file_sel_get_filename (GzFileSel *filesel)
{
    return filesel->filename;
}


gboolean
gz_file_sel_get_muli_dir_select (GzFileSel *filesel)
{
    return filesel->multi_dir_select;
}


gboolean
gz_file_sel_get_muli_file_select (GzFileSel *filesel)
{
    return filesel->multi_file_select;
}


GzFileSelType
gz_file_sel_get_selection_type (GzFileSel *filesel)
{
    return filesel->selection_type;
}


void
gz_file_sel_set_multi_dir_select (GzFileSel *filesel, gboolean value)
{
    filesel->multi_dir_select = value;
    if (value == TRUE)
        gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION (filesel)->dir_list),
          GTK_SELECTION_EXTENDED);
    else
        gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION (filesel)->dir_list),
          GTK_SELECTION_SINGLE);
}


void
gz_file_sel_set_multi_file_select (GzFileSel *filesel, gboolean value)
{
    filesel->multi_file_select = value;
    if (value == TRUE)
        gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION (filesel)->file_list),
          GTK_SELECTION_EXTENDED);
    else
        gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION (filesel)->file_list),
          GTK_SELECTION_SINGLE);
}


typedef struct
{
    guint select_signal, cancel_signal;
    gboolean result;
    GzFileSel *filesel;
    GMainLoop *loop;
} GzLoop;


static void
gz_file_sel_cancel_loop (GzLoop *loop)
{
    gtk_signal_disconnect (GTK_OBJECT (loop->filesel), loop->select_signal);
    gtk_signal_disconnect (GTK_OBJECT (loop->filesel), loop->cancel_signal);
    g_main_quit (loop->loop);
}


static void
gz_file_sel_files_selected (GzLoop *loop, gchar *filename, GList *files,
  gpointer user_data)
{
    gz_file_sel_cancel_loop (loop);
    loop->result = TRUE;
}


gboolean
gz_file_sel_run_and_close (GzFileSel *filesel)
{
    GzLoop *loop = g_new0 (GzLoop, 1);
    gboolean result;

    loop->filesel = filesel;
    loop->loop = g_main_new (FALSE);

    loop->select_signal = gtk_signal_connect_object (
      GTK_OBJECT (filesel),
      "select",
      GTK_SIGNAL_FUNC (gz_file_sel_files_selected),
      (GtkObject *) loop);
    loop->cancel_signal = gtk_signal_connect_object (
      GTK_OBJECT (filesel),
      "cancel",
      GTK_SIGNAL_FUNC (gz_file_sel_cancel_loop),
      (GtkObject *) loop);

    gtk_widget_show (GTK_WIDGET (filesel));
    g_main_run (loop->loop);

    result = loop->result;
    g_main_destroy (loop->loop);
    g_free (loop);

    return result;
}


void
gz_file_sel_set_selection_type (GzFileSel *filesel, GzFileSelType selection_type)
{
    filesel->selection_type = selection_type;
    switch (selection_type)
    {
    case GZ_FILE_SEL_TYPE_FILE_ONLY: case GZ_FILE_SEL_TYPE_FILE_AND_DIR:
        gtk_widget_set_sensitive (GTK_FILE_SELECTION (filesel)->file_list, TRUE);
        gtk_widget_set_sensitive (GTK_FILE_SELECTION (filesel)->dir_list, TRUE);
        break;
    case GZ_FILE_SEL_TYPE_DIR_ONLY:
        gtk_widget_set_sensitive (GTK_FILE_SELECTION (filesel)->file_list, FALSE);
        gtk_widget_set_sensitive (GTK_FILE_SELECTION (filesel)->dir_list, TRUE);
        break;
    }
}


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _FILESEL_C_