/*
 *  GnomeZip
 *  text-viewer.c - GnomeZip's internal text viewer
 *
 *  Copyright (c) 2000 Hongli Lai
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _TEXT_VIEWER_C_
#define _TEXT_VIEWER_C_

#include <gnome.h>
#include <misc.h>
#include <text-viewer.h>
#include <filesel.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


static void
text_viewer_init (TextViewer *obj, GladeXML *xml_data)
{
    obj->xml_data = xml_data;

    obj->window = glade_xml_get_widget (obj->xml_data, "textviewer");
    gtk_object_set_data (GTK_OBJECT (obj->window), "APP_STRUCT",
      (gpointer) obj);

    obj->less = glade_lookup_widget (obj->window, "less1");
}


guint
text_viewer_get_type (void)
{
    static guint ga_type = 0;

    if (!ga_type) {
        GtkTypeInfo ga_info = {
          "TextViewer",
          sizeof (TextViewer),
          sizeof (TextViewerClass),
          (GtkClassInitFunc) NULL,
          (GtkObjectInitFunc) NULL,
          (GtkArgSetFunc) NULL,
          (GtkArgGetFunc) NULL,
          (GtkClassInitFunc) NULL
        };
        ga_type = gtk_type_unique (gtk_object_get_type (), &ga_info);
    }
    return ga_type;
}


TextViewer *
text_viewer_new (GladeXML *xml_data)
{
    TextViewer *obj;

    obj = TEXT_VIEWER (gtk_type_new ((GtkType) TEXT_VIEWER_TYPE));
    text_viewer_init (obj, xml_data);
    return obj;
}


void
text_viewer_copy_text (GtkWidget *widget, gpointer user_data)
{
    GnomeLess *less = GNOME_LESS (glade_lookup_widget (widget, "less1"));
    GnomeApp *app = GNOME_APP (glade_lookup_widget (widget, "textviewer"));

    gtk_editable_copy_clipboard (GTK_EDITABLE (less->text));
    gnome_app_flash (app, _("Text copied to clipboard..."));
}


void
text_viewer_destroy (TextViewer *obj)
{
    g_return_if_fail (obj != NULL);
    g_return_if_fail (TEXT_IS_VIEWER (obj) == TRUE);

    if (obj->window != NULL) gtk_widget_destroy (obj->window);
    if (obj->filename != NULL) g_free (obj->filename);
    gtk_object_destroy (GTK_OBJECT (obj));
}


void
text_viewer_on_about (GtkWidget *widget)
{
    const gchar *authors[] = {
      "Hongli Lai <hongli@telekabel.nl",
      NULL
    };
    GtkWidget *about = gnome_about_new (
      N_("GnomeZip Text Viewer"),
      "0.2",
      N_("Copyright (c) 2000 Hongli Lai"),
      (const gchar **) authors,
      N_("GnomeZip's internal text viewer, intended to be fast and lightweight."),
      NULL
    );
    gtk_widget_show (about);
}


gboolean
text_viewer_on_delete (GtkWidget *widget, gpointer user_data)
{
    return TRUE;
}


void
text_viewer_open_file (TextViewer *tv, const gchar *filename)
{
    g_return_if_fail (tv != NULL);
    g_return_if_fail (TEXT_IS_VIEWER (tv) == TRUE);

    if (tv->filename != NULL) g_free (tv->filename);
    tv->filename = g_strdup (filename);
    gnome_less_show_file (GNOME_LESS (tv->less), filename);
    gtk_window_set_title (GTK_WINDOW (tv->window), g_strdup_printf (
      _("Text Viewer - %s"), filename));
}


void
text_viewer_open_with_command (GtkWidget *widget, const gchar *command,
  gboolean terminal)
{
    GtkObject *app = GTK_OBJECT (glade_lookup_widget (widget, "textviewer"));
    TextViewer *tv = TEXT_VIEWER (gtk_object_get_data (app, "APP_STRUCT"));
    gchar *cmd;

    if (terminal)
        cmd = g_strdup_printf ("gnome-terminal -e \"%s '%s'\" &", command, tv->filename);
    else
        cmd = g_strdup_printf ("%s '%s' &", command, tv->filename);

    gnome_execute_shell (NULL, cmd);
    g_free (cmd);
}


void
text_viewer_save (GtkWidget *widget, gpointer gdata)
{
    GtkObject *app = GTK_OBJECT (glade_lookup_widget (widget, "textviewer"));
    TextViewer *tv = TEXT_VIEWER (gtk_object_get_data (app, "APP_STRUCT"));
    GtkEditable *text = GTK_EDITABLE (GNOME_LESS (tv->less)->text);
    GzFileSel *filesel;

    int fd;
    gchar *buf;

    filesel = gz_file_sel_new (_("Save Text"));
    if (gz_file_sel_run_and_close (filesel))
    {
        buf = gtk_editable_get_chars (text, 0, -1);
        fd = creat (filesel->filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
        write (fd, buf, strlen (buf));
        close (fd);
    }
    gtk_widget_destroy (GTK_WIDGET (filesel));
}


/************** External editors ***************/

void
text_viewer_open_xemacs (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "xemacs", FALSE);
}

void
text_viewer_open_vi (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "vi", TRUE);
}

void
text_viewer_open_emacs (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "emacs", TRUE);
}

void
text_viewer_open_gedit (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "gedit", FALSE);
}

void
text_viewer_open_gvim (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "gvim", FALSE);
}

void
text_viewer_open_gless (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "gless", FALSE);
}

void
text_viewer_open_less (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "less", TRUE);
}

void
text_viewer_open_nedit (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "nedit", FALSE);
}

void
text_viewer_open_cooledit (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "cooledit", FALSE);
}

void
text_viewer_open_gnotepadp (GtkWidget *widget)
{
    text_viewer_open_with_command (widget, "gnp", FALSE);
}

static void
text_viewer_open_other_real (gchar *string, gpointer data)
{
    if (string == NULL) return;
    text_viewer_open_with_command ((GtkWidget *) data, string, TRUE);
}

void
text_viewer_open_other (GtkWidget *widget)
{
    GtkWindow *app = GTK_WINDOW (glade_lookup_widget (widget, "textviewer"));

    gnome_request_dialog (FALSE, _("Enter a command:"),
      NULL, PATH_MAX, text_viewer_open_other_real,
      widget, app);
}


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _TEXT_VIEWER_C_
